﻿local L = LibStub("AceLocale-3.0"):GetLocale("DKFight")

function DKFight:CreateFrame()
	-- Anchor
	self.Anchor = CreateFrame("Frame", "DKFightAnchor", UIParent)
	self.Anchor:SetResizable(true)
	self.Anchor:SetMinResize(60, 60)
	self.Anchor:SetMovable(true)
	self.Anchor:SetPoint("CENTER", UIParent, "CENTER")
	self.Anchor:SetHeight(self.db.profile.h)
	self.Anchor:SetWidth(self.db.profile.w)

    -- Display
	self.Display = CreateFrame("Frame", "DKFightDisplay", self.Anchor)
	self.Display:SetResizable(true)
	self.Display:EnableMouse(true)
	self.Display:SetPoint("TOPLEFT", self.Anchor, "TOPLEFT")
	self.Display:SetPoint("TOPRIGHT", self.Anchor, "TOPRIGHT")
	self.Display:SetPoint("BOTTOMLEFT", self.Anchor, "BOTTOMLEFT")
	self.Display:SetPoint("BOTTOMRIGHT", self.Anchor, "BOTTOMRIGHT")

	-- Background
    local bgFrame = {
		bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 6,
		insets = {left = 1, right = 1, top = 1, bottom = 1}
    }

	self.Display:SetBackdrop(bgFrame)
	self.Display:SetBackdropColor(self.db.profile.colors.background.r,
                                  	self.db.profile.colors.background.g,
                                  	self.db.profile.colors.background.b)
	self.Display:SetBackdropBorderColor(0,0,0)
	
	-- Events for scrolling & stopping sizing/moving.
	self.Display:SetScript("OnMouseDown", function(_, button)
            if button == "LeftButton" and not self.db.profile.locked then
                self.Anchor:StartMoving();
            end
        end)

	self.Display:SetScript("OnMouseUp", function()
            if not self.db.profile.locked then
		self.Anchor:StopMovingOrSizing();		
		self:SetAnchors()
            end
	end)

	-- Grip
	self.Grip = CreateFrame("Button", "DKFightResizeGrip", self.Display)
	self.Grip:SetNormalTexture("Interface\\AddOns\\DKFight\\ResizeGrip")
	self.Grip:SetHighlightTexture("Interface\\AddOns\\DKFight\\ResizeGrip")
	self.Grip:SetWidth(16)
	self.Grip:SetHeight(16)
	
	-- Events for resizing.
	self.Grip:SetScript("OnMouseDown", function()
		if not self.db.profile.locked then
			this:GetParent():GetParent():StartSizing()
			this:GetParent():GetParent().IsMovingOrSizing = true
		end
	end)
	
	self.Grip:SetScript("OnMouseUp", function()
		this:GetParent():GetParent():StopMovingOrSizing()
		this:GetParent():GetParent().IsMovingOrSizing = nil
		self:SetAnchors()
	end)
	
	self.Grip:SetPoint("BOTTOMRIGHT", self.Display, "BOTTOMRIGHT", 1, -1)
	
	--Cooldown Icons
	local x = 0
	local y = -5
	local padding = 5
	local size = 50
	local t = nil
	local count = 0
	for spell,details in pairs(self.spells) do
		if spell ~= details then
			x = (count * size) + padding
			details.displayFrame = CreateFrame("Cooldown", details.frameName, self.Display)
			details.displayFrame:SetWidth(size)
			details.displayFrame:SetHeight(size)
			details.displayFrame:SetPoint("TOPLEFT", x, y)
			-- Texture
			t = details.displayFrame:CreateTexture(nil,"BACKGROUND")
			t:SetTexture(details.texturePath)
			t:SetAllPoints(details.displayFrame)
			t:SetAlpha(.8)
			details.displayFrame.texture = t
			details.texture = t
			count = count + 1
		end
	end
	
	-- Update the display
	self:UpdateDisplay()
	self.Anchor:SetScript("OnUpdate", function(this, elapsed)
		self:OnDisplayUpdate(elapsed)
	end)
end

function DKFight:OnDisplayUpdate(elapsed)
--self.timeSinceDKFight = self.timeSinceDKFight + elapsed
		--self.Display:SetAlpha(self.db.profile.colors.background.a * (1 - ((self.timeSinceDKFight - self.db.profile.colors.faded.a)/5)))
	--Cooldown Icons
	local x = 0
	local y = -5
	local padding = 5
	local size = 50
	local t = nil
	local count = 0
	
	for spell,details in pairs(self.spells) do
		if spell ~= details then
			if details.state then
				x = (count * size) + padding
				details.displayFrame:SetAlpha(1)
				details.displayFrame:SetPoint("TOPLEFT", x, y)
				count = count + 1
			else
				details.displayFrame:SetAlpha(0)
			end
		end
	end
end

function DKFight:SetAnchors()
	local x, y, height, width, relativePoint, relativeTo = nil,nil,nil,nil,nil,nil

    height, width = self.Anchor:GetHeight(), self.Anchor:GetWidth()
    relTo,_,relPt,x,y = self.Anchor:GetPoint()

	self.Anchor:ClearAllPoints()
    self.Anchor:SetPoint(relTo, UIParent, relPt, x, y)
	self.Anchor:SetHeight(height)
	self.Anchor:SetWidth(width)

	self.db.profile.x = x
	self.db.profile.y = y
	self.db.profile.h = h
	self.db.profile.w = w
	self.db.profile.relTo = relTo
	self.db.profile.relPt = relPt
end

function DKFight:UpdateDisplay()
	if not self.Anchor or not self.db.profile.show then return end

	self.Anchor:SetScale(self.db.profile.scale)
	self.Anchor:SetFrameStrata(self.db.profile.strata)

	if self.db.profile.locked then
		self.Grip:Hide()
	else
		self.Grip:Show()
	end

	local color = self.db.profile.colors.background;
	self.Display:SetBackdropColor(color.r, color.g, color.b)
end

function DKFight:UpdateVisible()
	if not self.Anchor then return end

	if self.db.profile.enabled and self.db.profile.show then
		self.Anchor:Show()
	else
		self.Anchor:Hide()
	end
end